@@ -145,7 +145,7 @@ class Contact(Entity):
             - status
                 * Filter messages by status
                 * Allowed values: ignored, processing, received, sent, queued, failed,
-                    failed_queued, cancelled, delivered, not_delivered
+                    failed_queued, cancelled, delivered, not_delivered, read
             
             - time_created[min] (UNIX timestamp)
                 * Filter messages created on or after a particular time
@@ -155,18 +155,26 @@ class Contact(Entity):
             
             - external_id
                 * Filter messages by ID from an external provider
+                * Allowed modifiers: external_id[ne], external_id[exists]
             
             - contact_id
                 * ID of the contact who sent/received the message
+                * Allowed modifiers: contact_id[ne], contact_id[exists]
             
             - phone_id
                 * ID of the phone (basic route) that sent/received the message
             
             - broadcast_id
                 * ID of the broadcast containing the message
+                * Allowed modifiers: broadcast_id[ne], broadcast_id[exists]
             
             - scheduled_id
                 * ID of the scheduled message that created this message
+                * Allowed modifiers: scheduled_id[ne], scheduled_id[exists]
+            
+            - group_id
+                * Filter messages sent or received by contacts in a particular group. The group must
+                    be a normal group, not a dynamic group.
             
             - sort
                 * Sort the results based on a field
@@ -243,16 +251,15 @@ class Contact(Entity):
             
             - time_created (UNIX timestamp)
                 * Filter scheduled messages by time_created
-                * Allowed modifiers: time_created[ne], time_created[min], time_created[max]
+                * Allowed modifiers: time_created[min], time_created[max]
             
             - next_time (UNIX timestamp)
                 * Filter scheduled messages by next_time
-                * Allowed modifiers: next_time[ne], next_time[min], next_time[max],
-                    next_time[exists]
+                * Allowed modifiers: next_time[min], next_time[max], next_time[exists]
             
             - sort
                 * Sort the results based on a field
-                * Allowed values: default, name
+                * Allowed values: default, next_time
                 * Default: default
             
             - sort_dir
@@ -282,7 +289,7 @@ class Contact(Entity):
             
             - time_created (UNIX timestamp)
                 * Filter data rows by the time they were created
-                * Allowed modifiers: time_created[ne], time_created[min], time_created[max]
+                * Allowed modifiers: time_created[min], time_created[max]
             
             - sort
                 * Sort the results based on a field
