@@ -55,24 +55,22 @@ class Group(Entity):
             
             - time_created (UNIX timestamp)
                 * Filter contacts by time created
-                * Allowed modifiers: time_created[ne], time_created[min], time_created[max]
+                * Allowed modifiers: time_created[min], time_created[max]
             
             - last_message_time (UNIX timestamp)
                 * Filter contacts by last time a message was sent or received
-                * Allowed modifiers: last_message_time[ne], last_message_time[min],
-                    last_message_time[max], last_message_time[exists]
+                * Allowed modifiers: last_message_time[min], last_message_time[max],
+                    last_message_time[exists]
             
             - last_incoming_message_time (UNIX timestamp)
                 * Filter contacts by last time a message was received
-                * Allowed modifiers: last_incoming_message_time[ne],
-                    last_incoming_message_time[min], last_incoming_message_time[max],
-                    last_incoming_message_time[exists]
+                * Allowed modifiers: last_incoming_message_time[min],
+                    last_incoming_message_time[max], last_incoming_message_time[exists]
             
             - last_outgoing_message_time (UNIX timestamp)
                 * Filter contacts by last time a message was sent
-                * Allowed modifiers: last_outgoing_message_time[ne],
-                    last_outgoing_message_time[min], last_outgoing_message_time[max],
-                    last_outgoing_message_time[exists]
+                * Allowed modifiers: last_outgoing_message_time[min],
+                    last_outgoing_message_time[max], last_outgoing_message_time[exists]
             
             - incoming_message_count (int)
                 * Filter contacts by number of messages received from the contact
@@ -129,16 +127,15 @@ class Group(Entity):
             
             - time_created (UNIX timestamp)
                 * Filter scheduled messages by time_created
-                * Allowed modifiers: time_created[ne], time_created[min], time_created[max]
+                * Allowed modifiers: time_created[min], time_created[max]
             
             - next_time (UNIX timestamp)
                 * Filter scheduled messages by next_time
-                * Allowed modifiers: next_time[ne], next_time[min], next_time[max],
-                    next_time[exists]
+                * Allowed modifiers: next_time[min], next_time[max], next_time[exists]
             
             - sort
                 * Sort the results based on a field
-                * Allowed values: default, name
+                * Allowed values: default, next_time
                 * Default: default
             
             - sort_dir
