@@ -83,11 +83,31 @@ class Project(Entity):
                     short URLs.
                 * Default: false
             
+            - short_link_params (dict)
+                *
+                    If `track_clicks` is true, `short_link_params` may be used to specify
+                    custom parameters for each short link in the message. The following parameters are
+                    supported:
+                    
+                    `domain` (string): A custom short domain name to use for the short
+                    links. The domain name must already be registered for your project or organization.
+                    
+                    `expiration_sec` (integer): The number of seconds after the message is
+                    created (queued to send) when the short links will stop forwarding to the
+                    destination URL.
+                    If null, the short links will not expire.
+            
             - media_urls (array)
                 * URLs of media files to attach to the text message. If `message_type` is `sms`,
                     short links to each media URL will be appended to the end of the content (separated
                     by a new line).
             
+            - route_params (dict)
+                * Route-specific parameters for the message. The parameters object should have one
+                    or more keys matching the `phone_type` field of a phone (basic route) that may be
+                    used to send the message. The corresponding value should be an object with
+                    route-specific parameters to use if the message is sent by that type of route.
+            
             - label_ids (array)
                 * List of IDs of labels to add to this message
             
@@ -201,6 +221,20 @@ class Project(Entity):
                     short URLs.
                 * Default: false
             
+            - short_link_params (dict)
+                *
+                    If `track_clicks` is true, `short_link_params` may be used to specify
+                    custom parameters for each short link in the message. The following parameters are
+                    supported:
+                    
+                    `domain` (string): A custom short domain name to use for the short
+                    links. The domain name must already be registered for your project or organization.
+                    
+                    `expiration_sec` (integer): The number of seconds after the message is
+                    created (queued to send) when the short links will stop forwarding to the
+                    destination URL.
+                    If null, the short links will not expire.
+            
             - media_urls (array)
                 * URLs of media files to attach to the text message. If `message_type` is `sms`,
                     short links to each URL will be appended to the end of the content (separated by a
@@ -209,6 +243,12 @@ class Project(Entity):
             - vars (dict)
                 * Custom variables to set for each message
             
+            - route_params (dict)
+                * Route-specific parameters for the messages in the broadcast. The parameters object
+                    may have keys matching the `phone_type` field of a phone (basic route) that may be
+                    used to send messages in this broadcast. The corresponding value is an object with
+                    route-specific parameters to use when sending messages with that type of route.
+            
             - service_id
                 * Service to invoke for each recipient (when `message_type` is `call` or `service`)
                 * Required if message_type is service
@@ -292,11 +332,36 @@ class Project(Entity):
                     available variables)](#variables)
                 * Default: false
             
+            - track_clicks (boolean)
+                * If true, URLs in the message content will automatically be replaced with unique
+                    short URLs.
+                * Default: false
+            
+            - short_link_params (dict)
+                *
+                    If `track_clicks` is true, `short_link_params` may be used to specify
+                    custom parameters for each short link in the message. The following parameters are
+                    supported:
+                    
+                    `domain` (string): A custom short domain name to use for the short
+                    links. The domain name must already be registered for your project or organization.
+                    
+                    `expiration_sec` (integer): The number of seconds after the message is
+                    created (queued to send) when the short links will stop forwarding to the
+                    destination URL.
+                    If null, the short links will not expire.
+            
             - media_urls (array)
                 * URLs of media files to attach to the text message. If `message_type` is `sms`,
                     short links to each media URL will be appended to the end of the content (separated
                     by a new line).
             
+            - route_params (dict)
+                * Route-specific parameters to apply to all messages. The parameters object may have
+                    keys matching the `phone_type` field of a phone (basic route) that may be used to
+                    send messages. The corresponding value is an object with route-specific parameters
+                    to use when sending messages with that type of route.
+            
             - vars (dict)
                 * Custom variables to store with the message
             
@@ -322,6 +387,10 @@ class Project(Entity):
                       (Other properties of the Message object are
                       omitted in order to reduce the amount of redundant data sent in each API
                       response.)
+                      If the `messages` parameter in the API request
+                      contains items with `to_number` values that are associated with blocked contacts,
+                      the `id` and `status` properties corresponding to those items will be null, and no
+                      messages will be sent to those numbers.
               
               - broadcast_id
                   * ID of broadcast that these messages are associated with, if `broadcast_id` or
@@ -440,6 +509,20 @@ class Project(Entity):
                     short URLs.
                 * Default: false
             
+            - short_link_params (dict)
+                *
+                    If `track_clicks` is true, `short_link_params` may be used to specify
+                    custom parameters for each short link in the message. The following parameters are
+                    supported:
+                    
+                    `domain` (string): A custom short domain name to use for the short
+                    links. The domain name must already be registered for your project or organization.
+                    
+                    `expiration_sec` (integer): The number of seconds after the message is
+                    created (queued to send) when the short links will stop forwarding to the
+                    destination URL.
+                    If null, the short links will not expire.
+            
             - is_template (bool)
                 * Set to true to evaluate variables like [[contact.name]] in message content
                 * Default: false
@@ -449,6 +532,12 @@ class Project(Entity):
                     short links to each media URL will be appended to the end of the content (separated
                     by a new line).
             
+            - route_params (dict)
+                * Route-specific parameters to use when sending the message. The parameters object
+                    may have keys matching the `phone_type` field of a phone (basic route) that may be
+                    used to send the message. The corresponding value is an object with route-specific
+                    parameters to use when sending a message with that type of route.
+            
             - label_ids (array)
                 * Array of IDs of labels to add to the sent messages (maximum 5). Does not apply
                     when `message_type`=`service`, since the labels are determined by the service
@@ -649,24 +738,22 @@ class Project(Entity):
             
             - time_created (UNIX timestamp)
                 * Filter contacts by time created
-                * Allowed modifiers: time_created[ne], time_created[min], time_created[max]
+                * Allowed modifiers: time_created[min], time_created[max]
             
             - last_message_time (UNIX timestamp)
                 * Filter contacts by last time a message was sent or received
-                * Allowed modifiers: last_message_time[ne], last_message_time[min],
-                    last_message_time[max], last_message_time[exists]
+                * Allowed modifiers: last_message_time[min], last_message_time[max],
+                    last_message_time[exists]
             
             - last_incoming_message_time (UNIX timestamp)
                 * Filter contacts by last time a message was received
-                * Allowed modifiers: last_incoming_message_time[ne],
-                    last_incoming_message_time[min], last_incoming_message_time[max],
-                    last_incoming_message_time[exists]
+                * Allowed modifiers: last_incoming_message_time[min],
+                    last_incoming_message_time[max], last_incoming_message_time[exists]
             
             - last_outgoing_message_time (UNIX timestamp)
                 * Filter contacts by last time a message was sent
-                * Allowed modifiers: last_outgoing_message_time[ne],
-                    last_outgoing_message_time[min], last_outgoing_message_time[max],
-                    last_outgoing_message_time[exists]
+                * Allowed modifiers: last_outgoing_message_time[min],
+                    last_outgoing_message_time[max], last_outgoing_message_time[exists]
             
             - incoming_message_count (int)
                 * Filter contacts by number of messages received from the contact
@@ -760,8 +847,8 @@ class Project(Entity):
             
             - last_active_time (UNIX timestamp)
                 * Filter phones by last active time
-                * Allowed modifiers: last_active_time[ne], last_active_time[min],
-                    last_active_time[max], last_active_time[exists]
+                * Allowed modifiers: last_active_time[min], last_active_time[max],
+                    last_active_time[exists]
             
             - sort
                 * Sort the results based on a field
@@ -845,7 +932,7 @@ class Project(Entity):
             - status
                 * Filter messages by status
                 * Allowed values: ignored, processing, received, sent, queued, failed,
-                    failed_queued, cancelled, delivered, not_delivered
+                    failed_queued, cancelled, delivered, not_delivered, read
             
             - time_created[min] (UNIX timestamp)
                 * Filter messages created on or after a particular time
@@ -855,18 +942,26 @@ class Project(Entity):
             
             - external_id
                 * Filter messages by ID from an external provider
+                * Allowed modifiers: external_id[ne], external_id[exists]
             
             - contact_id
                 * ID of the contact who sent/received the message
+                * Allowed modifiers: contact_id[ne], contact_id[exists]
             
             - phone_id
                 * ID of the phone (basic route) that sent/received the message
             
             - broadcast_id
                 * ID of the broadcast containing the message
+                * Allowed modifiers: broadcast_id[ne], broadcast_id[exists]
             
             - scheduled_id
                 * ID of the scheduled message that created this message
+                * Allowed modifiers: scheduled_id[ne], scheduled_id[exists]
+            
+            - group_id
+                * Filter messages sent or received by contacts in a particular group. The group must
+                    be a normal group, not a dynamic group.
             
             - sort
                 * Sort the results based on a field
@@ -1069,8 +1164,16 @@ class Project(Entity):
                     <table>
                     <tr><td> `service_id` </td> <td> The ID of the
                     service to apply (string) </td></tr>
+                    <tr><td> `variables` </td> <td> Optional object
+                    containing up to 25 temporary variable names and their corresponding values to set
+                    when invoking the service. Values may be strings, numbers, or boolean (true/false).
+                    String values may be up to 4096 bytes in length. Arrays and objects are not
+                    supported. Within Custom Actions, each variable can be used like [[$name]] (with a
+                    leading $ character and surrounded by double square brackets). Within a Cloud Script
+                    API service or JavaScript action, each variable will be available as a global
+                    JavaScript variable like $name (with a leading $ character). (object) </td></tr>
                     </table>
-                    
+                    <br />
                     **`update_contact_var`**, **`update_message_var`**,
                     **`update_row_var`**:
                     <table>
@@ -1079,50 +1182,50 @@ class Project(Entity):
                     <tr><td> `value` </td> <td> The value to set
                     (string, boolean, float, null) </td></tr>
                     </table>
-                    
+                    <br />
                     **`add_group_members`**, **`remove_group_members`**:
                     <table>
                     <tr><td> `group_id` </td> <td> The ID of the group
                     (string) </td></tr>
                     </table>
-                    
+                    <br />
                     **`add_label`**, **`remove_label`**:
                     <table>
                     <tr><td> `label_id` </td> <td> The ID of the label
                     (string) </td></tr>
                     </table>
-                    
+                    <br />
                     **`resend_messages`**:
                     <table>
                     <tr><td> `route_id` </td> <td> ID of the new route
                     to use, or null to use the original route (string) </td></tr>
                     </table>
-                    
+                    <br />
                     **`set_send_blocked`**:
                     <table>
                     <tr><td> `send_blocked` </td> <td> `true` to block
                     sending messages, `false` to unblock sending messages (boolean) </td></tr>
                     </table>
-                    
+                    <br />
                     **`set_conversation_status`**:
                     <table>
                     <tr><td> `conversation_status` </td> <td> "active",
                     "handled", or "closed" (string) </td></tr>
                     </table>
-                    
+                    <br />
                     **`export_contacts`**, **`export_messages`**,
                     **`export_rows`**:
                     <table>
                     <tr><td>`storage_id` </td> <td> ID of a storage
-                    backend where the CSV file will be saved. (string)
+                    provider where the CSV file will be saved. (string)
                     
                     Currently only AWS S3 is supported as a storage
-                    backend.
+                    provider.
                     This requires creating a S3 bucket in your own
                     AWS account, as well as an IAM user with access key and secret that has permission
                     to write to that bucket.
-                    To configure your own S3 bucket as a storage
-                    backend, contact support.
+                    You can configure your own S3 bucket as a
+                    storage provider on the <a href="/dashboard/a/storage">Storage Providers</a> page.
                     
                     Direct downloads are not supported when
                     exporting data via the API.
@@ -1133,7 +1236,7 @@ class Project(Entity):
                     save in the CSV file. If not provided, all default columns will be saved. (array of
                     strings, optional) </td></tr>
                     </table>
-                    
+                    <br />
                     **`delete_contacts`**, **`delete_messages`**,
                     **`delete_rows`**, **`cancel_messages`**, **`retry_message_services`**: <br />
                     No parameters.
@@ -1527,16 +1630,15 @@ class Project(Entity):
             
             - time_created (UNIX timestamp)
                 * Filter scheduled messages by time_created
-                * Allowed modifiers: time_created[ne], time_created[min], time_created[max]
+                * Allowed modifiers: time_created[min], time_created[max]
             
             - next_time (UNIX timestamp)
                 * Filter scheduled messages by next_time
-                * Allowed modifiers: next_time[ne], next_time[min], next_time[max],
-                    next_time[exists]
+                * Allowed modifiers: next_time[min], next_time[max], next_time[exists]
             
             - sort
                 * Sort the results based on a field
-                * Allowed values: default, name
+                * Allowed values: default, next_time
                 * Default: default
             
             - sort_dir
@@ -1811,6 +1913,187 @@ class Project(Entity):
         from .airtimetransaction import AirtimeTransaction
         return AirtimeTransaction(self._api, {'project_id': self.id, 'id': id}, False)
 
+    def getContactFields(self):
+        """
+        Gets a list of all custom fields defined for contacts in this project. The return value is
+        an array of objects with the properties 'name', 'variable', 'type', 'order', 'readonly', and
+        'lookup_key'. (Fields are automatically created any time a Contact's 'vars' property is
+        updated.)
+        
+        Returns:
+            array
+        """
+        return self._api.doRequest("GET", self.getBaseApiPath() + "/contact_fields")
+
+    def setContactFieldMetadata(self, variable, **options):
+        """
+        Allows customizing how a custom contact field is displayed in the Telerivet web app.
+        
+        Arguments:
+          - variable
+              * The variable name of the field to create or update.
+              * Required
+          
+              * Required
+            
+            - name (string, max 64 characters)
+                * Display name for the field
+            
+            - type (int)
+                * Field type
+                * Allowed values: text, long_text, phone_number, email, url, audio, date, date_time,
+                    number, boolean, select
+            
+            - order (int)
+                * Order in which to display the field
+            
+            - items (array)
+                * Array of up to 100 objects containing `value` and `label` string properties to
+                    show in the dropdown list when type is `select`. Each `value` and `label` must be
+                    between 1 and 256 characters in length.
+                * Required if type is `select`
+            
+            - readonly (bool)
+                * Set to true to prevent editing the field in the Telerivet web app
+            
+            - lookup_key (bool)
+                * Set to true to allow using this field as a lookup key when importing contacts via
+                    the Telerivet web app
+            
+            - show_on_conversation (bool)
+                * Set to true to show field on Conversations tab
+          
+        Returns:
+            object
+        """
+        return self._api.doRequest("POST", self.getBaseApiPath() + "/contact_fields/%s" % (variable), options)
+
+    def getMessageFields(self):
+        """
+        Gets a list of all custom fields defined for messages in this project. The return value is
+        an array of objects with the properties 'name', 'variable', 'type', 'order', 'readonly', and
+        'lookup_key'. (Fields are automatically created any time a Contact's 'vars' property is
+        updated.)
+        
+        Returns:
+            array
+        """
+        return self._api.doRequest("GET", self.getBaseApiPath() + "/message_fields")
+
+    def setMessageFieldMetadata(self, variable, **options):
+        """
+        Allows customizing how a custom message field is displayed in the Telerivet web app.
+        
+        Arguments:
+          - variable
+              * The variable name of the field to create or update.
+              * Required
+          
+              * Required
+            
+            - name (string, max 64 characters)
+                * Display name for the field
+            
+            - type (string)
+                * Field type
+                * Allowed values: text, long_text, phone_number, email, url, audio, date, date_time,
+                    number, boolean, select
+            
+            - order (int)
+                * Order in which to display the field
+            
+            - items (array)
+                * Array of up to 100 objects containing `value` and `label` string properties to
+                    show in the dropdown list when type is `select`. Each `value` and `label` must be
+                    between 1 and 256 characters in length.
+                * Required if type is `select`
+            
+            - hide_values (bool)
+                * Set to true to avoid showing values of this field on the Messages page
+          
+        Returns:
+            object
+        """
+        return self._api.doRequest("POST", self.getBaseApiPath() + "/message_fields/%s" % (variable), options)
+
+    def getMessageStats(self, **options):
+        """
+        Retrieves statistics about messages sent or received via Telerivet. This endpoint returns
+        historical data that is computed shortly after midnight each day in the project's time zone,
+        and does not contain message statistics for the current day.
+        
+        Arguments:
+              * Required
+            
+            - start_date (string)
+                * Start date of message statistics, in YYYY-MM-DD format
+                * Required
+            
+            - end_date (string)
+                * End date of message statistics (inclusive), in YYYY-MM-DD format
+                * Required
+            
+            - rollup (string)
+                * Date interval to group by
+                * Allowed values: day, week, month, year, all
+                * Default: day
+            
+            - properties (string)
+                * Comma separated list of properties to group by
+                * Allowed values: org_id, org_name, org_industry, project_id, project_name, user_id,
+                    user_email, user_name, phone_id, phone_name, phone_type, direction, source, status,
+                    network_code, network_name, message_type, service_id, service_name, simulated, link
+            
+            - metrics (string)
+                * Comma separated list of metrics to return (summed for each distinct value of the
+                    requested properties)
+                * Allowed values: count, num_parts, duration, price
+                * Required
+            
+            - currency (string)
+                * Three-letter ISO 4217 currency code used when returning the 'price' field. If the
+                    original price was in a different currency, it will be converted to the requested
+                    currency using the approximate current exchange rate.
+                * Default: USD
+            
+            - filters (dict)
+                * Key-value pairs of properties and corresponding values; the returned statistics
+                    will only include messages where the property matches the provided value. Only the
+                    following properties are supported for filters: `user_id`, `phone_id`, `direction`,
+                    `source`, `status`, `service_id`, `simulated`, `message_type`, `network_code`
+          
+        Returns:
+            (associative array)
+              - intervals (array)
+                  * List of objects representing each date interval containing at least one message
+                      matching the filters.
+                      Each object has the following properties:
+                      
+                      <table>
+                      <tr><td> start_time </td> <td> The UNIX timestamp of the start
+                      of the interval (int) </td></tr>
+                      <tr><td> end_time </td> <td> The UNIX timestamp of the end of
+                      the interval, exclusive (int) </td></tr>
+                      <tr><td> start_date </td> <td> The date of the start of the
+                      interval in YYYY-MM-DD format (string) </td></tr>
+                      <tr><td> end_date </td> <td> The date of the end of the
+                      interval in YYYY-MM-DD format, inclusive (string) </td></tr>
+                      <tr><td> groups </td> <td> Array of groups for each
+                      combination of requested property values matching the filters (array)
+                      <br /><br />
+                      Each object has the following properties:
+                      <table>
+                      <tr><td> properties </td> <td> An object of key/value
+                      pairs for each distinct value of the requested properties (object) </td></tr>
+                      <tr><td> metrics </td> <td> An object of key/value pairs
+                      for each requested metric (object) </td></tr>
+                      </table>
+                      </td></tr>
+                      </table>
+        """
+        data = self._api.doRequest("GET", self.getBaseApiPath() + "/message_stats", options)
+        return data
+
     def save(self):
         """
         Saves any fields or custom variables that have changed for the project.
